from api_descstats_csv.utils import process_csv
import os
import json

filename = r"data/dataset_subset.csv"
path_json_expected = os.path.join(r"data/", "json_test_data.json")

histogram  = 10
b_mean = True
b_max = True
b_std = True
b_hist = True

json_res = process_csv(filename= filename,
            histogram= histogram,
            b_mean = True,
            b_max = True,
            b_std = True,
            b_hist = True)


with open(path_json_expected, "w", encoding="utf-8") as f:
    json.dump(json_res, f, ensure_ascii=False, indent=4)


import json
import os

import typer
from rich.console import Console
from utils import process_csv

app = typer.Typer(
    name="api_descstats_csv",
    help="Calculate simple descreptive stats from csv with Python with cli or import respecting the limit of 512 MB ",
    add_completion=False,
)

console = Console()


@app.command()
def main(
    filename: str = typer.Option(..., help="Path to readable csv."),
    histogram: int = typer.Option(..., help="Size of histogram bins."),
    output_path: str = typer.Option(..., help="Path to save metrics as json."),
    b_mean: bool = typer.Option(False, help="Determines if mean will be calculated."),
    b_max: bool = typer.Option(False, help="Determines if max will be calculated."),
    b_std: bool = typer.Option(False, help="Determines if std will be calculated."),
    b_hist: bool = typer.Option(False, help="Determines if histogram will be calculated."),
) -> None:

    console.print(f"[bold yellow]Processing each column of CSV[/]")

    data = process_csv(filename, histogram, b_mean, b_max, b_std, b_hist)

    data = {"columns": data}

    # Persist results
    path_json = os.path.join(output_path, "json_data.json")

    with open(path_json, "w", encoding="utf-8") as f:
        json.dump(data, f, ensure_ascii=False, indent=4)

    console.print(f"[bold green]Results saved in {output_path}")


if __name__ == "__main__":
    app()



import csv
import numpy as np
import pandas as pd
from tqdm import tqdm


def sizeof_fmt(num: float, suffix: str = "B") -> str:
    """by Fred Cirera,  https://stackoverflow.com/a/1094933/1870254, modified"""
    for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
        if abs(num) < 1024.0:
            return f"{num:3.1f} {unit}{suffix}"
        num /= 1024.0
    return "{:.1f} {}{}".format(num, "Yi", suffix)


def get_header(filename: str) -> list["str"]:
    with open(filename) as csvfile:
        datareader = csv.reader(csvfile)
        return next(datareader)


def read_col(filename: str, col: str) -> np.ndarray:
    arr = pd.read_csv(filename, engine="pyarrow", usecols=[col])
    arr = arr.values
    arr = arr[~np.isnan(arr)]
    arr = arr.astype(np.float32)
    return arr


def calculate_metrics(
    arr: np.dtype,
    col: str,
    b_mean: bool = False,
    b_max: bool = False,
    b_std: bool = False,
    b_hist: bool = False,
    sz: int = 10,
) -> dict:

    selected_metrics = {"name": col}

    # Calculate simple metrics (mean, std, max)
    simple_metrics = [
        ["mean", b_mean, np.mean],
        ["max", b_max, np.max],
        ["std", b_std, np.std],
    ]

    for v in simple_metrics:
        if v[1] == True:
            metric = round(v[2](arr), 2).astype(str)
            selected_metrics.update({v[0]: metric})

    # Calculate histogram
    bins = int(arr.shape[0] / sz)
    if b_hist == True:
        if bins < 3:
            raise ValueError("Histogram with less than 3 bins, try a smaller bin size")
        elif bins > 2 * 3840:
            raise ValueError(f"Too many bins, try a bigger bin size. Suggestion size: {int((arr.shape[0]/(2*3840))+2)}")

        hist, bin_edges = np.histogram(arr, bins=bins)

        selected_metrics.update(
            {
                "histogram": [
                    {
                        "from": round(bin_edges[i], 2).astype(str),
                        "to": round(bin_edges[i + 1], 2).astype(str),
                        "count": hist[i].astype(str),
                    }
                    for i in range(len(hist))
                ]
            }
        )

    del arr

    return selected_metrics


def process_csv(
    filename: str,
    histogram: int,
    b_mean: bool = False,
    b_max: bool = False,
    b_std: bool = False,
    b_hist: bool = False,
) -> dict:

    headers = get_header(filename)
    headers.remove("timestamp")

    data = []

    for col in tqdm(headers):

        # Load
        arr = read_col(filename, col)

        # Calculate and format results
        metrics_dict = calculate_metrics(
            arr=arr, col=col, b_mean=b_mean, b_max=b_max, b_std=b_std, b_hist=b_hist, sz=histogram
        )

        # Save results
        data.append(metrics_dict)
        del arr
        del metrics_dict

    return data


def metrics2pandas(
    filename: str, histogram: int, b_mean: bool = False, b_max: bool = False, b_std: bool = False, b_hist: bool = False
) -> pd.DataFrame:
    data = process_csv(filename, histogram=histogram, b_mean=b_mean, b_max=b_max, b_std=b_std, b_hist=b_hist)
    data = pd.DataFrame(data)

    return data
